import axios from 'axios'
import toCamel from 'camelcase-keys'
import React, { useEffect, useState } from 'react'
import { Link, useNavigate, useParams } from 'react-router-dom'

import TagSearch from '@/components/TagSearch'
import SectionTitle from '@/components/common/SectionTitle'
import SidebarComponent from '@/components/layout/SidebarComponent'
import { API_BASE_URL } from '@/config'

import type { Post, Tag } from '@/types'

type TagByCategory = { [key: string]: Tag[] }

type Props = { posts: Post[] }


export default ({ posts }: Props) => {
  const navigate = useNavigate ()

  const [tags, setTags] = useState<TagByCategory> ({ })

  useEffect (() => {
    const tagsTmp: TagByCategory = { }
    for (const post of posts)
      {
        for (const tag of post.tags)
          {
            if (!(tag.category in tagsTmp))
              tagsTmp[tag.category] = []
            if (!(tagsTmp[tag.category].map (t => t.id).includes (tag.id)))
              tagsTmp[tag.category].push (tag)
          }
      }
    for (const cat of Object.keys (tagsTmp))
      tagsTmp[cat].sort ((tagA, tagB) => tagA.name < tagB.name ? -1 : 1)
    setTags (tagsTmp)
  }, [posts])

  return (
      <SidebarComponent>
        <TagSearch />
        <SectionTitle>タグ</SectionTitle>
        <ul>
          {['general', 'deerjikist', 'nico'].map (cat => cat in tags && (
              <>
                {tags[cat].map (tag => (
                    <li key={tag.id} className="mb-1">
                      <Link to={`/posts?${ (new URLSearchParams ({ tags: tag.name })).toString () }`}>
                        {tag.name}
                      </Link>
                      <span className="ml-1">{tag.postCount}</span>
                    </li>))}
              </>))}
        </ul>
        <SectionTitle>関聯</SectionTitle>
        <a href="#"
           onClick={ev => {
             ev.preventDefault ()
             void (axios.get (`${ API_BASE_URL }/posts/random`)
                   .then (res => navigate (`/posts/${ res.data.id }`)))
           }}>
          ランダム
        </a>
      </SidebarComponent>)
}
